
/*

      Teensy 3.5, 120 Mhz, Faster


   Notes:

    - To use the "Libraries" you have to set the Sketchbook Location the Preferences inside the Arduino IDE (Teensy Project Folder)
      and use the "Teensyduino" IDE directly for Teensy Boards

    - If no Light Sensor is installed/actived, then the "RTC Time" will be set Day/Night
    - To set the Time in 1 minute ranges, wait 0.5 sec after the last button press

    - Teensy 3.5    cut the solder pads bridge at the back to prevent back current from USB 5V
    - Speaker       use a "LSM-S2514K" Speaker (the small ones have bad sound or are not working), use pluggable wires
    - OLED 1309     Maybe: use SPI insteat of I2C, because of random I2C Display problems

    - The soundfile on the SDCard has to be named: "alert.wav"
      (by the way, not all SDCards are working! So, if you can not hear a sound after a few trys, use another SDCard)

*/



// **************************************
// **************************************
// ***                                ***
// *** Activate the available Sensors ***
// ***                                ***
// **************************************
// **************************************

// AIR Sensors
#define SENSOR_SCD30
//#define SENSOR_MHZ19

// LIGHT Sensors
//#define SENSOR_BH1750
#define SENSOR_TSL2561

// **************************************
// **************************************





#include <malloc.h>
#include <EEPROM.h>

unsigned int updateInterval      = 5;  // min. 30 seconds waittime MAD : auf 5 geänderr

int dayLedBrightness    = 30;
int nightLedBrightness  = 10;
int displayBrightness   = 100;
boolean nightDisplay    = true;
boolean daySound        = true;
boolean nightSound      = true;
int airGoodValue        = 400;
int airBadValue         = 2400;
int airAlarmValue       = 2000;
int dayStartHour        = 9;
int dayEndHour          = 21;

int lightDayThreshold   = 10;
int16_t lightSensorOffset = 0;  // use int16_t insteat of "int", because "int" is unsigned int and "signed int" is not working!
int16_t airSensorOffset = 0;  // use int16_t insteat of "int", because "int" is unsigned int and "signed int" is not working!
int16_t tempSensorOffset = 0;  // use int16_t insteat of "int", because "int" is unsigned int and "signed int" is not working!
int16_t humSensorOffset = 0;  // use int16_t insteat of "int", because "int" is unsigned int and "signed int" is not working!


int badAir_LedNrStart   = 7;  // Green-Red spread, here you can adjust from which LEDnr the Air is going to bad:-)


// * Global Variables, do not edit *
int airValue            = 400;
int airTem              = 20;
int airHum              = 50;
int lightValue          = -1; // Default: N/A
int ledBrightness       = dayLedBrightness;



// *** Air Sensor SCD30 ***
#ifdef SENSOR_SCD30
#include "SparkFun_SCD30_Arduino_Library.h"
SCD30 airSensor;
#endif

// *** Air Sensor MHZ-19 ***
#ifdef SENSOR_MHZ19
#include <Mhz19.h>
Mhz19 airSensor;
#endif


// *** Light Sensor ***
#ifdef SENSOR_BH1750
#include <BH1750.h>
BH1750 lightMeter;
#endif

// *** Light Sensor TSL2561 ***
#ifdef SENSOR_TSL2561
#include <SparkFunTSL2561.h>
SFE_TSL2561 lightMeter;
boolean TSL2561_gain = false;
unsigned int TSL2561_ms;
unsigned char TSL2561_time = 2;
#endif


// *** RGB LEDS 2812 ***
#include <Adafruit_NeoPixel.h>
#define PIN            33
#define NUMPIXELS      16
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);


// *** LED 1309 Display ***
#include <U8g2lib.h>

#define USE_SPI 1           // 0 = I2C, 1 = SPI

//#define WIRE_PORT Wire      // Used if USE_SPI == 0
#define SPI_PORT SPI        // Used if USE_SPI == 1
#define RES_PIN 25
#define DC_PIN 26           // Used only if USE_SPI == 1
#define CS_PIN 10           // Used only if USE_SPI == 1

#if USE_SPI
//U8G2_SSD1309_128X64_NONAME2_F_4W_HW_SPI u8g2(U8G2_R0, /* cs=*/ 10, /* dc=*/ 26, /* reset=*/ 25);                                // Hardware SPI
U8G2_SSD1309_128X64_NONAME0_F_4W_SW_SPI u8g2(U8G2_R0, /* clock=*/ 27, /* data=*/ 28, /* cs=*/ 10, /* dc=*/ 26, /* reset=*/ 25);   // Software SPI

#else
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);   // All Boards without Reset of the Display - SPI I2c IIc Ebay bords
#endif


// *** Buttons ***
#define BUTTON_LEFT     7
#define BUTTON_MIDDLE   9
#define BUTTON_RIGHT    5


// *** LED ***
#define LED_PIN    13


// *** AUDIO ***
#include <Audio.h>

AudioPlaySdWav           playWav1;
AudioOutputAnalogStereo  audioOutput;
AudioConnection          patchCord1(playWav1, 0, audioOutput, 0);
AudioConnection          patchCord2(playWav1, 1, audioOutput, 1);
// GUItool: end automatically generated code


// *** SD Card ***
#include <SD.h>
#define SDCARD_CS_PIN BUILTIN_SDCARD
boolean SDInserted = false;


// *** RTC ***
#include <TimeLib.h>
time_t RTCTime;


// *** MENU ***
const int STATE_IDLE            = 0;
const int STATE_READY_TO_OPEN   = 1;
const int STATE_OPENED          = 2;

int menuState = STATE_IDLE;
int submenu = 0;
// MAD: Temperatur und Luftfeuchtigkeits Sensor Offset ins Menu gebastelt 
String submenuNames[] =  {  "UPDATE INTERVALL", "DISLPAY CONTRAST", "LIGHT THRESHOLD", "LIGHT SENS OFFSET", "BRIGHTNESS @ DAY",
                            "BRIGHTNESS @ NIGHT", "DISLPAY @ NIGHT", "SOUND @ DAY", "SOUND @ NIGHT", "AIR VERY GOOD VALUE",
                            "AIR VERY BAD VALUE", "AIR ALARM VALUE", "AIR SENS OFFSET", "TEMP SENS OFFSET", "HUM SENS OFFSET" ,"DAY START HOUR", "DAY END HOUR", "CLOCK", "EXIT"
                         };

int submenuNamesAmount = 19;
int EXIT_VALUE = submenuNamesAmount - 1;



int displayXPosStart = 7;
int displayYPosStart = 15;

void setup() {

  loadValues();


  pinMode(BUTTON_LEFT, INPUT_PULLUP);
  pinMode(BUTTON_MIDDLE, INPUT_PULLUP);
  pinMode(BUTTON_RIGHT, INPUT_PULLUP);

  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW);

  if ((SD.begin(SDCARD_CS_PIN))) SDInserted = true;

  AudioMemory(12);

  u8g2.begin();
  u8g2.setBusClock(400000); // Slower Speed
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.clearBuffer();

  Wire.begin();
  Wire.setClock(400000);

  pixels.begin();
  clearRGBLEDs();

#ifdef SENSOR_BH1750
  lightMeter.begin();
#endif


#ifdef SENSOR_TSL2561
  if (lightMeter.begin() == false) {
    Serial.println("Light Sensor TSL2561 not detected. Please check wiring. Freezing...");
    while (1);
  }
  else {
    lightMeter.setTiming(TSL2561_gain, TSL2561_time, TSL2561_ms);
    lightMeter.setPowerUp();
  }
#endif


#ifdef SENSOR_SCD30
  if (airSensor.begin() == false) {
    Serial.println("Air sensor SCD-30 not detected. Please check wiring. Freezing...");
    while (1);
  }
  else {
    /*
      delay(1000);
      airSensor.setAutoSelfCalibration(1);
      delay(100);
      airSensor.setMeasurementInterval(30);
    */
  }
#endif

#ifdef SENSOR_MHZ19
  Serial1.begin(9600);
  airSensor.begin(&Serial1);
  airSensor.setMeasuringRange(Mhz19MeasuringRange::Ppm_5000);
  airSensor.enableAutoBaseCalibration();
  if (airSensor.isReady() == false) {
    Serial.println("Air sensor MHZ-19 not detected. Please check wiring. Freezing...");
    while (1);
  }
#endif

  delay(3000);



  // *** Menu Button ***
  attachInterrupt(digitalPinToInterrupt(BUTTON_LEFT), button1Event, LOW);

}


void loop() {

  updateSensorValues();

  useBrightnessValues();

  updateRGBLeds(false);

  updateOLEDDisplay();

  checkToPlayAlertSound();


  // * Draw OLED Display Progress Effect *
  long t = millis();
  while (millis() - t < updateInterval * 1000) {

    // if (isDay() || nightDisplay == true) { // Draw Progress Rectangle
    if (isDay()) { // Draw Progress Rectangle MAD : der Progress Balken wird nur Tagsüber aufgebaut 

      drawProgress(displayXPosStart, displayYPosStart + 30, 106, 4, updateInterval * 1000, millis() - t);
    }
    //yield();
    delay(25);

    checkNotDayButtons();
  }
}

boolean showDisplayOnce = false;

void checkNotDayButtons() {

  if (!isDay() && menuState == STATE_IDLE) {
    if (digitalRead(BUTTON_MIDDLE) == LOW || digitalRead(BUTTON_RIGHT) == LOW) {

      // Wait for button release
      while (digitalRead(BUTTON_MIDDLE) == LOW || digitalRead(BUTTON_RIGHT) == LOW) delay(10);

      // Show Display
      showDisplayOnce = true;
      ledBrightness = dayLedBrightness;
      updateRGBLeds(true);
      updateOLEDDisplay();
      showDisplayOnce = false;
    }
  }
}


void updateOLEDDisplay() {

  u8g2.clearDisplay();

  if (isDay() || nightDisplay == true || showDisplayOnce) {

    if (isDay()) drawLine(displayXPosStart, displayYPosStart + 24, 106); // MAD: Trennlinie im Night Modus aus
    u8g2.sendBuffer();

    updateAirText(displayXPosStart, displayYPosStart + 2);
    updateLightText(displayXPosStart, displayYPosStart + 16);
    updateInfoText(displayXPosStart, displayYPosStart + 46);
  }

 // if (isDay() || nightDisplay == true)  drawProgressFrame(displayXPosStart, displayYPosStart + 30, 106, 4);
  if (isDay())  drawProgressFrame(displayXPosStart, displayYPosStart + 30, 106, 4); // MAD: zeigt den Progress Rahmen nur am Tag an 

}


long lastButton1Event = millis();

void button1Event() {
  if (millis() - lastButton1Event < 200) return;
  lastButton1Event = millis();

  if (menuState == STATE_IDLE) menuButtonEvent();
}

void _delay(unsigned int ms) {

  long t = millis();
  while (millis() - t < ms) {
    yield();
  }
}

void drawLine(int x, int y, int len) {
  u8g2.drawLine(x, y, x + len, y);
}

void drawProgress(int x, int y, int w, int h, int timeDelay, int time) {
  u8g2.drawFrame(x + ((float)w / timeDelay * time), y,  1, h);
  u8g2.sendBuffer();
}

void drawProgressFrame(int x, int y, int w, int h) {
  u8g2.drawFrame(x, y, w + 1, h);
  u8g2.sendBuffer();
}

void clearProgress(int x, int y, int w, int h) {
  u8g2.drawFrame(x, y, w + 1, h);
  u8g2.sendBuffer();
}

void updateLightText(int x, int y) {

  u8g2.setCursor(x, y);
  u8g2.print("LIGHT SENSOR: ");

#if defined(SENSOR_BH1750) || defined (SENSOR_TSL2561)
  if (lightValue >= 0) u8g2.print(lightValue);
  else                u8g2.print("TIME");
#else
  u8g2.print("TIME");
#endif
  u8g2.sendBuffer();
}

void updateAirText(int x, int y) {

  u8g2.setCursor(x, y);
  u8g2.print("AIR QUALITY : ");
// u8g2.print("AIR off.");        // MAD: zu testzwecken geändert - orig  --- u8g2.print("AIR QUALITY : "); ---
//  u8g2.print(airSensorOffset);  // MAD: zu testzwecken hinzugefügt
//  u8g2.print(" :");             // MAD: zu testzwecken hinzugefügt
  u8g2.sendBuffer();
  u8g2.print(airValue);
  u8g2.sendBuffer();
}

void updateInfoText(int x, int y) {

  u8g2.setCursor(x, y);
  u8g2.print(airTem);
  u8g2.print(F("\260"));      // MAD: "\260" . für das Grad C Zeichen eingefügt 
  u8g2.print("C");
  u8g2.setCursor(x + 43, y);
  u8g2.print(airHum);
  u8g2.print("%");

  u8g2.setCursor(x + 78, y);
  printClock();

  u8g2.sendBuffer();
}

void printClock() {
  time_t t = Teensy3Clock.get();
  u8g2.print(twoDigits(hour(t)));
  u8g2.print(":");
  u8g2.print(twoDigits(minute(t)));
}

String twoDigits(String s) {
  if (s.length() < 2) s = "0" + s;
  return s;
}


void updateSensorValues() {

  int val = 0;
  int tem = 0;
  int hum = 0;

#ifdef SENSOR_SCD30
  delay(100);
  if (airSensor.dataAvailable()) {
    val = airSensor.getCO2();
    tem = airSensor.getTemperature();
    hum = airSensor.getHumidity();
  }
#endif

#ifdef SENSOR_MHZ19
  delay(100);
  val = airSensor.getCarbonDioxide();

  while (val == 0) {
    _delay(20000);
    val = airSensor.getCarbonDioxide();
    tem = airSensor.getTemperature();
    // hum = sensor.getHumidity();
  }
#endif


  airValue = val + airSensorOffset;
  airTem = tem + tempSensorOffset;  //MAD : Temperatur Offset hinzugefügt
  airHum = hum + humSensorOffset;  // MAD : Luftfeuchtigkeit Offset hinzugefügt 

#ifdef SENSOR_BH1750
  lightValue =  (int)lightMeter.readLightLevel() + lightSensorOffset;
  return lux;
#else
  lightValue = -1;
#endif

#ifdef SENSOR_TSL2561
  unsigned int data0, data1;
  if (lightMeter.getData(data0, data1)) {

    double lux;
    boolean good = lightMeter.getLux(TSL2561_gain, TSL2561_ms, data0, data1, lux);
    if (good)  lightValue = (int) lux + lightSensorOffset;
    else       lightValue = 255;
  }
#else
  lightValue = -1;
#endif
}


void checkToPlayAlertSound() {
  if ((isDay() && !daySound) || (!isDay() && !nightSound)) return;

  if (SDInserted && airValue >= airAlarmValue) {
    if (playWav1.isPlaying() == false)playWav1.play("ALERT.WAV");
  }
}

boolean isDay() {

  boolean dayValue = true;

  if (lightValue == -1) { // If no Light Sensor available... use the Time

    time_t t = Teensy3Clock.get();
    int h = hour(t);

    if (h >= dayStartHour && h <= dayEndHour) { // DAY
      dayValue = true;
    }
    else {
      dayValue = false;
    }
  }
  else {

#if defined(SENSOR_BH1750) || defined (SENSOR_TSL2561)
    if (lightValue >= 0 && lightValue < lightDayThreshold) dayValue =  false; // Night
#endif
  }

  return dayValue;
}


float yellowCounter = 0;
float redCounter = 0;

void updateRGBLeds(boolean fast) {

  int singleLED_range = airBadValue / 16;
  int lightUpLEDCount = airValue / singleLED_range;

  float yellowCounter = 0;
  float redCounter    = 0;

  for (int i = 0; i < lightUpLEDCount; i++) {

    if (i <= badAir_LedNrStart)  yellowCounter += (float)ledBrightness / (badAir_LedNrStart + 3);
    else                         redCounter    += (float)ledBrightness / (12 - badAir_LedNrStart);

    if (i < badAir_LedNrStart)   pixels.setPixelColor(i, pixels.Color(yellowCounter, ledBrightness, 0)); // 0-255
    else                         pixels.setPixelColor(i, pixels.Color(yellowCounter, ledBrightness - redCounter, 0)); // 0-255
    pixels.show();

    if (!fast) _delay(50);
  }

  // * All upper Leds Off *
  for (int i = lightUpLEDCount; i < 16; i++) pixels.setPixelColor(i, pixels.Color(0, 0, 0));
  pixels.show();
}

void clearRGBLEDs() {
  for (int i = 0; i < 16; i++)  pixels.setPixelColor(i, pixels.Color(0, 0, 0));  // 0-255
  pixels.show();
}


boolean editMode = false;

void menuButtonEvent() {

  if (menuState == STATE_IDLE) {

    menuState = STATE_OPENED;
    submenu = 0;
    updateOptionsMenu();

    doSubmenu();
  }
}



void doSubmenu() {

  boolean submenuFirstEnter = true;

  while (menuState == STATE_OPENED) {

    if (digitalRead(BUTTON_LEFT) == LOW)   {

      while (digitalRead(BUTTON_LEFT) == LOW) {};

      if (submenu == EXIT_VALUE) {
        saveValues();

        updateOLEDDisplay();

        menuState = STATE_IDLE;
        submenu = 0;
        lastButton1Event = millis();

        return;
      }
      else {
        if (!submenuFirstEnter) subMenuEventLeft();
      }
    }
    else if (digitalRead(BUTTON_MIDDLE) == LOW)   {
      subMenuEventMiddle();
      long t = millis();
      while (digitalRead(BUTTON_MIDDLE) == LOW) {
        if (millis() - t > 200) break;
      };
    }
    else  if (digitalRead(BUTTON_RIGHT) == LOW) {
      subMenuEventRight();
      long t = millis();
      while (digitalRead(BUTTON_RIGHT) == LOW) {
        if (millis() - t > 200) break;
      };
    }

    submenuFirstEnter = false;
    delay(20);
  }
}

void subMenuEventLeft() {
  if (menuState != STATE_OPENED) return;

  while (digitalRead(BUTTON_LEFT) == LOW) {};

  if (!editMode) {

    editMode = true;
    updateOptionsMenu();
  }
  else {

    editMode = false;
    updateOptionsMenu();
  }

}
void subMenuEventMiddle() {
  if (menuState != STATE_OPENED) return;

  if (!editMode) {

    submenu--;
    if (submenu < 0) submenu = submenuNamesAmount - 1;
    updateOptionsMenu();
  }
  else {
    changeValues(1);
  }
}

void subMenuEventRight() {
  if (menuState != STATE_OPENED) return;

  if (!editMode) {
    submenu++;
    if (submenu >= submenuNamesAmount) submenu = 0;
    updateOptionsMenu();
  }
  else {
    changeValues(2);
  }
}


void updateOptionsMenu() {
  if (menuState != STATE_OPENED) return;

  u8g2.clearDisplay();
  u8g2.setCursor(10, 20);
  u8g2.print(submenuNames[submenu]);
  u8g2.sendBuffer();

  // Exlude "Exit"
  if (submenu != EXIT_VALUE) {
    u8g2.setCursor(10, 35);

    showOptions();
    if (editMode) showEditOptions();
    else {
      u8g2.setFont(u8g2_font_micro_tr);

      int w = (int)((float)100 / submenuNamesAmount);

      u8g2.drawBox(10 + submenu * w, 5, w, 2);

      u8g2.setCursor(31, 56);
      u8g2.print("EDIT");
      u8g2.setCursor(60, 56);
      u8g2.print("<-");
      u8g2.setCursor(87, 56);
      u8g2.print("->");

      u8g2.setFont(u8g2_font_6x10_tf);
    }
    u8g2.sendBuffer();
  }
}


void showOptions() {

  switch (submenu) {

    case 0:
      if (updateInterval < 60) {
        u8g2.print(updateInterval);
        u8g2.print(" sec");
      } else {
        u8g2.print(updateInterval / 60);
        u8g2.print(" min");
      }
      break;
    case 1:
      u8g2.print(displayBrightness);
      break;
    case 2:
      u8g2.print(lightDayThreshold);
      break;
    case 3:
      u8g2.print(lightSensorOffset);
      break;
    case 4:
      u8g2.print(dayLedBrightness);
      break;
    case 5:
      u8g2.print(nightLedBrightness);
      break;
    case 6:
      u8g2.print(convertBooleanToYesNo(nightDisplay));
      break;
    case 7:
      u8g2.print(convertBooleanToYesNo(daySound));
      break;
    case 8:
      u8g2.print(convertBooleanToYesNo(nightSound));
      break;
    case 9:
      u8g2.print(airGoodValue);
      break;
    case 10:
      u8g2.print(airBadValue);
      break;
    case 11:
      u8g2.print(airAlarmValue);
      break;
    case 12:
      u8g2.print(airSensorOffset);
      break;
 // MAD: Temperatur und Luftfeuchtigkeit Offset Menu hinzugefügt  
    case 13:
      u8g2.print(tempSensorOffset);
      break;
    case 14:
      u8g2.print(humSensorOffset);
      break;
    case 15:
      u8g2.print(dayStartHour);
      break;
    case 16:
      u8g2.print(dayEndHour);
      break;
    case 17:
      printClock();
      break;
    default:
      break;
  }
}

void showEditOptions() {

  u8g2.setFont(u8g2_font_micro_tr); // Small Font
  u8g2.setCursor(30, 57);
  u8g2.print("BACK");

  u8g2.setCursor(63, 57);
  u8g2.print("-");

  u8g2.setCursor(89, 57);
  u8g2.print("+");

  u8g2.sendBuffer();
  u8g2.setFont(u8g2_font_6x10_tf); // Normal Font
}

char * convertBooleanToYesNo(boolean state) {
  char * yes = strdup("YES");
  char * no = strdup("NO");

  if (state) return yes;
  else       return no;
}


long lastChangeTime = millis();

void changeValues(int buttonMiddleRight) {

  if (buttonMiddleRight == 1) {     // Middle Button pressed

    switch (submenu) {
      case 0:
        if (updateInterval > 60)      updateInterval -= 60;
        else if (updateInterval > 5) updateInterval -= 5;  // min. 30 Seconds MAD: geaendert auf 5 sek Schritte vorher 30/10 jetzt 5/5
        break;
      case 1:
        if (displayBrightness > 10)    {
          displayBrightness -= 10;
          useBrightnessValues();
        }
        break;
      case 2:
        if (lightDayThreshold > 1)    lightDayThreshold -= 1;
        break;
      case 3:
        if (lightSensorOffset > -254)    lightSensorOffset -= 1;
        break;
      case 4:
        if (dayLedBrightness >= 5)    {
          dayLedBrightness -= 5;
          ledBrightness = dayLedBrightness;
          updateRGBLeds(true);
        }
        break;
      case 5:
        if (nightLedBrightness >= 1) {
          nightLedBrightness -= 1;
          ledBrightness = nightLedBrightness;
          updateRGBLeds(true);
        }
        break;
      case 6:
        nightDisplay = false;
        break;
      case 7:
        daySound = false;
        break;
      case 8:
        nightSound = false;
        break;
      case 9:
        if (airGoodValue > 400)    airGoodValue -= 100;
        break;
      case 10:
        if (airBadValue > 1000)    airBadValue -= 100;
        break;
      case 11:
        if (airAlarmValue > 500)    airAlarmValue -= 100;
        break;
      case 12:
        if (airSensorOffset > -2000)    airSensorOffset -= 10;
        break;
// MAD: Temperatur und Luftfeuchtigkeit Offset Abfrage hinzugefügt  
      case 13:
        if (tempSensorOffset > -20)    tempSensorOffset -= 1;
        break;
      case 14:
        if (humSensorOffset > -20)    humSensorOffset -= 1;
        break;
      case 15:
        if (dayStartHour > 0)    dayStartHour -= 1;
        break;
      case 16:
        if (dayEndHour > 0)    dayEndHour -= 1;
        break;
      case 17:
        time_t t = Teensy3Clock.get() - 60;
        if (millis() - lastChangeTime < 400)  t = Teensy3Clock.get() - 60 * 5;
        Teensy3Clock.set(t); // set the RTC
        setTime(t);
        lastChangeTime = millis();
        break;
    }
  }
  else if (buttonMiddleRight == 2) { // Right Button pressed

    switch (submenu) {
      case 0:
        if (updateInterval < 60)         updateInterval += 10;
        else if (updateInterval <= 540)  updateInterval += 60;
        break;
      case 1:
        if (displayBrightness < 255)  {
          displayBrightness += 5;
          useBrightnessValues();
        }
        break;
      case 2:
        if (lightDayThreshold < 254)    lightDayThreshold += 1;
        break;
      case 3:
        if (lightSensorOffset < 254)    lightSensorOffset += 1;
        break;
      case 4:
        if (dayLedBrightness < 255)     {
          dayLedBrightness += 5;
          ledBrightness = dayLedBrightness;
          updateRGBLeds(true);
        }
        break;
      case 5:
        if (nightLedBrightness < 255) {
          nightLedBrightness += 5;
          ledBrightness = nightLedBrightness;
          updateRGBLeds(true);
        }
        break;
      case 6:
        nightDisplay = true;
        break;
      case 7:
        daySound = true;
        break;
      case 8:
        nightSound = true;
        break;
      case 9:
        if (airGoodValue < 1000)    airGoodValue += 100;
        break;
      case 10:
        if (airBadValue < 3000)    airBadValue += 100;
        break;
      case 11:
        if (airAlarmValue < 3000)    airAlarmValue += 100;
        break;
      case 12:
        if (airSensorOffset < 2000)    airSensorOffset += 20;
        break;
// MAD: Temperatur und Luftfeuchtigkeit Offset Abfrage hinzugefügt  
      case 13:
        if (tempSensorOffset < 20)    tempSensorOffset += 1;
        break;
      case 14:
        if (humSensorOffset < 20)    humSensorOffset += 1;
        break;
      case 15:
        if (dayStartHour < 24)    dayStartHour += 1;
        break;
      case 16:
        if (dayEndHour < 24)    dayEndHour += 1;
        break;
      case 17:
        time_t t = Teensy3Clock.get() + 60;
        if (millis() - lastChangeTime < 400)  t = Teensy3Clock.get() + 60 * 5;
        Teensy3Clock.set(t); // set the RTC
        setTime(t);
        lastChangeTime = millis();
        break;
    }
  }

  updateOptionsMenu();
}

void loadValues() {

  if (EEPROM.read(100) != 1 || EEPROM.read(101) != 2 || EEPROM.read(102) != 3) {

    saveValues();
  }
  else {

    updateInterval        = EEPROM.read(110) & 0xff;
    dayLedBrightness      = EEPROM.read(111) & 0xff;
    nightLedBrightness    = EEPROM.read(112) & 0xff;
    displayBrightness     = EEPROM.read(113) & 0xff;
    nightDisplay          = EEPROM.read(114) & 0xff;
    daySound              = EEPROM.read(115) & 0xff;
    nightSound            = EEPROM.read(116) & 0xff;
    airGoodValue          = ((EEPROM.read(117) & 0xff) << 8) | (EEPROM.read(118) & 0xff);
    airBadValue           = ((EEPROM.read(119) & 0xff) << 8) | (EEPROM.read(120) & 0xff);
    airAlarmValue         = ((EEPROM.read(121) & 0xff) << 8) | (EEPROM.read(122) & 0xff);

    lightDayThreshold = EEPROM.read(123) & 0xff;
    if (lightDayThreshold == 255) lightDayThreshold = 10;

    lightSensorOffset = ((EEPROM.read(124) & 0xff) << 8) | (EEPROM.read(125) & 0xff);
    if (lightSensorOffset < -254 || lightSensorOffset > 254) lightSensorOffset = 0;

    dayStartHour          = EEPROM.read(126) & 0xff;
    if (dayStartHour < 0 || dayStartHour > 24) dayStartHour = 9;
    dayEndHour            = EEPROM.read(127) & 0xff;
    if (dayEndHour < 0 || dayEndHour > 24) dayEndHour = 21;

    airSensorOffset = ((EEPROM.read(128) & 0xff) << 8) | (EEPROM.read(129) & 0xff);
    if (airSensorOffset < -2000 || airSensorOffset > 2000) airSensorOffset = 0;
    if (airSensorOffset  == -1) airSensorOffset = 0;

// MAD: Temperatur und Luftfeuchtigkeit Offset hinzugefügt  

    tempSensorOffset = ((EEPROM.read(130) & 0xff) << 8) | (EEPROM.read(131) & 0xff);
    if (tempSensorOffset < -20 || tempSensorOffset > 20) tempSensorOffset = 0;
    if (tempSensorOffset  == -1) tempSensorOffset = 0;

    humSensorOffset = ((EEPROM.read(132) & 0xff) << 8) | (EEPROM.read(133) & 0xff);
    if (humSensorOffset < -20 || humSensorOffset > 20) humSensorOffset = 0;
    if (humSensorOffset  == -1) humSensorOffset = 0;


    if (updateInterval < 10) updateInterval = 10; // min. 30 Seconds MAD geändert auf 10 Sek

  }
}

void saveValues() {

  EEPROM.write(100, 1);
  EEPROM.write(101, 2);
  EEPROM.write(102, 3);

  EEPROM.write(110, updateInterval);
  EEPROM.write(111, dayLedBrightness);
  EEPROM.write(112, nightLedBrightness);
  EEPROM.write(113, displayBrightness);
  EEPROM.write(114, nightDisplay);
  EEPROM.write(115, daySound);
  EEPROM.write(116, nightSound);
  EEPROM.write(117, airGoodValue >> 8);
  EEPROM.write(118, airGoodValue & 0xff);
  EEPROM.write(119, airBadValue >> 8);
  EEPROM.write(120, airBadValue & 0xff);
  EEPROM.write(121, airAlarmValue >> 8);
  EEPROM.write(122, airAlarmValue & 0xff);
  EEPROM.write(123, lightDayThreshold & 0xff);
  EEPROM.write(124, lightSensorOffset >> 8);
  EEPROM.write(125, lightSensorOffset & 0xff);
  EEPROM.write(126, dayStartHour);
  EEPROM.write(127, dayEndHour);
  EEPROM.write(128, airSensorOffset >> 8);
  EEPROM.write(129, airSensorOffset & 0xff);
  //MAD: Temperatur und Luftfeuchtigkeit Offset ins EEProm schreiben 
  EEPROM.write(130, tempSensorOffset >> 8);
  EEPROM.write(131, tempSensorOffset & 0xff);
  EEPROM.write(132, humSensorOffset >> 8);
  EEPROM.write(133, humSensorOffset & 0xff);

  useBrightnessValues();
}

void useBrightnessValues() {

  u8g2.setContrast(displayBrightness);

  if (isDay()) ledBrightness = dayLedBrightness;
  else         ledBrightness = nightLedBrightness;

}
